@@ -1,5 +1,6 @@
+from .load_records import Zone
 from .main import DNSServer
 from .version import VERSION
 
-__all__ = 'DNSServer', '__version__'
+__all__ = 'DNSServer', 'Zone', '__version__'
 __version__ = VERSION
