@@ -1,10 +1,11 @@
 from typing import Any, Callable, Dict, List
 
+import dns
 import pytest
 from dirty_equals import IsIP, IsPositive
 from dns.resolver import NoAnswer, Resolver as RawResolver
 
-from dnserver import DNSServer
+from dnserver import DNSServer, Zone
 
 Resolver = Callable[[str, str], List[Dict[str, Any]]]
 
@@ -33,17 +34,21 @@ def convert_answer(answer) -> Dict[str, Any]:
 
 
 @pytest.fixture(scope='session')
-def dns_resolver():
+def dns_server():
     port = 5053
 
-    server = DNSServer('example_zones.toml', port=port)
+    server = DNSServer.from_toml('example_zones.toml', port=port)
     server.start()
-
     assert server.is_running
+    yield server
+    server.stop
 
+
+@pytest.fixture(scope='session')
+def dns_resolver(dns_server: DNSServer):
     resolver = RawResolver()
     resolver.nameservers = ['127.0.0.1']
-    resolver.port = port
+    resolver.port = dns_server.port
 
     def resolve(name: str, type_: str) -> List[Dict[str, Any]]:
         answers = resolver.resolve(name, type_)
@@ -51,8 +56,6 @@ def dns_resolver():
 
     yield resolve
 
-    server.stop()
-
 
 def test_a_record(dns_resolver: Resolver):
     assert dns_resolver('example.com', 'A') == [
@@ -133,7 +136,7 @@ def test_soa_higher(dns_resolver: Resolver):
 def test_dns_server_without_upstream():
     port = 5054
 
-    server = DNSServer('example_zones.toml', port=port, upstream=None)
+    server = DNSServer.from_toml('example_zones.toml', port=port, upstream=None)
     server.start()
 
     resolver = RawResolver()
@@ -155,3 +158,61 @@ def test_dns_server_without_upstream():
             resolve('python.org', 'A')
     finally:
         server.stop()
+
+
+def test_dynamic_zone_update(dns_server: DNSServer, dns_resolver: Resolver):
+    assert dns_resolver('example.com', 'A') == [
+        {
+            'type': 'A',
+            'value': '1.2.3.4',
+        },
+    ]
+    with pytest.raises(dns.resolver.NXDOMAIN):
+        dns_resolver('another-example.org', 'A')
+
+    dns_server.add_record(Zone(host='another-example.com', type='A', answer='2.3.4.5'))
+
+    assert dns_resolver('example.com', 'A') == [
+        {
+            'type': 'A',
+            'value': '1.2.3.4',
+        },
+    ]
+    assert dns_resolver('another-example.com', 'A') == [
+        {
+            'type': 'A',
+            'value': '2.3.4.5',
+        },
+    ]
+
+    dns_server.set_records([Zone(host='example.com', type='A', answer='4.5.6.7')])
+
+    assert dns_resolver('example.com', 'A') == [
+        {
+            'type': 'A',
+            'value': '4.5.6.7',
+        },
+    ]
+    with pytest.raises(dns.resolver.NXDOMAIN):
+        dns_resolver('another-example.org', 'A')
+
+
+def test_no_zone_at_initialization():
+    port = 5055
+
+    server = DNSServer(port=port, upstream=None)
+    server.start()
+
+    resolver = RawResolver()
+    resolver.nameservers = ['127.0.0.1']
+    resolver.port = port
+
+    def resolve(name: str, type_: str) -> List[Dict[str, Any]]:
+        answers = resolver.resolve(name, type_)
+        return [convert_answer(answer) for answer in answers]
+
+    try:
+        with pytest.raises(NoAnswer):
+            resolve('example.com', 'A')
+    finally:
+        server.stop()
